@@ -1,4 +1,3 @@
-
 /**
  * SPDX-FileCopyrightText: 2021-2022 Bart De Vries <bart@mogwai.be>
  *
@@ -7,10 +6,12 @@
 
 #include "updatefeedjob.h"
 
+#include <QCryptographicHash>
 #include <QDir>
 #include <QDomElement>
 #include <QMultiMap>
 #include <QNetworkReply>
+#include <QSqlError>
 #include <QSqlQuery>
 #include <QTextDocumentFragment>
 #include <QTimer>
@@ -19,26 +20,29 @@
 #include <ThreadWeaver/Thread>
 
 #include "database.h"
+#include "datatypes.h"
 #include "enclosure.h"
+#include "error.h"
 #include "fetcher.h"
 #include "fetcherlogging.h"
-#include "kasts-version.h"
 #include "settingsmanager.h"
 #include "storagemanager.h"
 
 using namespace ThreadWeaver;
+using namespace DataTypes;
 
-UpdateFeedJob::UpdateFeedJob(const QString &url, const QByteArray &data, QObject *parent)
+UpdateFeedJob::UpdateFeedJob(const QString &url, const QByteArray &data, const FeedDetails &feed, QObject *parent)
     : QObject(parent)
     , m_url(url)
     , m_data(data)
+    , m_feed(feed)
 {
     // connect to signals in Fetcher such that GUI can pick up the changes
     connect(this, &UpdateFeedJob::feedDetailsUpdated, &Fetcher::instance(), &Fetcher::feedDetailsUpdated);
     connect(this, &UpdateFeedJob::feedUpdated, &Fetcher::instance(), &Fetcher::feedUpdated);
     connect(this, &UpdateFeedJob::entryAdded, &Fetcher::instance(), &Fetcher::entryAdded);
     connect(this, &UpdateFeedJob::entryUpdated, &Fetcher::instance(), &Fetcher::entryUpdated);
-    connect(this, &UpdateFeedJob::feedUpdateStatusChanged, &Fetcher::instance(), &Fetcher::feedUpdateStatusChanged);
+    connect(this, &UpdateFeedJob::error, &Fetcher::instance(), &Fetcher::error);
 }
 
 void UpdateFeedJob::run(JobPointer, Thread *)
@@ -50,31 +54,9 @@ void UpdateFeedJob::run(JobPointer, Thread *)
 
     Database::openDatabase(m_url);
 
-    // First check if the RSS file has changed since last update; we do this by
-    // comparing to the SHA1 hash we saved last time
-    bool skipUpdate = false;
-
-    m_newHash = QString::fromLatin1(QCryptographicHash::hash(m_data, QCryptographicHash::Sha256).toHex());
-
-    QSqlQuery query(QSqlDatabase::database(m_url));
-    query.prepare(QStringLiteral("SELECT lastHash FROM Feeds WHERE url=:url;"));
-    query.bindValue(QStringLiteral(":url"), m_url);
-    Database::execute(query);
-    if (query.next()) {
-        m_oldHash = query.value(QStringLiteral("lastHash")).toString();
-        qCDebug(kastsFetcher) << "RSS hashes (old and new)" << m_url << m_oldHash << m_newHash;
-        if (m_newHash == m_oldHash) {
-            skipUpdate = true;
-            qCDebug(kastsFetcher) << "same RSS feed hash as last time; skipping feed update for" << m_url;
-        }
-    }
-    query.clear(); // release lock on database
-
-    if (!skipUpdate) {
-        Syndication::DocumentSource document(m_data, m_url);
-        Syndication::FeedPtr feed = Syndication::parserCollection()->parse(document, QStringLiteral("Atom"));
-        processFeed(feed);
-    }
+    Syndication::DocumentSource document(m_data, m_url);
+    Syndication::FeedPtr feed = Syndication::parserCollection()->parse(document, QStringLiteral("Atom"));
+    processFeed(feed);
 
     Database::closeDatabase(m_url);
 
@@ -89,70 +71,64 @@ void UpdateFeedJob::processFeed(Syndication::FeedPtr feed)
         return;
 
     // First check if this is a newly added feed and get current name and dirname
-    m_isNewFeed = false;
-    QString oldName, oldDirname;
-    QSqlQuery query(QSqlDatabase::database(m_url));
-    query.prepare(QStringLiteral("SELECT new, name, dirname FROM Feeds WHERE url=:url;"));
-    query.bindValue(QStringLiteral(":url"), m_url);
-    Database::execute(query);
-    if (query.next()) {
-        m_isNewFeed = query.value(QStringLiteral("new")).toBool();
-        oldName = query.value(QStringLiteral("name")).toString();
-        oldDirname = query.value(QStringLiteral("dirname")).toString();
-    } else {
-        qCDebug(kastsFetcher) << "Feed not found in database" << m_url;
-        return;
-    }
-    if (m_isNewFeed) {
+    if (m_feed.isNew) {
         qCDebug(kastsFetcher) << "New feed" << feed->title();
     }
 
     m_markUnreadOnNewFeed = !(SettingsManager::self()->markUnreadOnNewFeed() == 2);
+    QDateTime current = QDateTime::currentDateTime();
+
+    m_updateFeed = m_feed; // start from current feed details in database
+
+    m_updateFeed.name = feed->title();
+    m_updateFeed.link = feed->link();
+    m_updateFeed.description = feed->description();
+    m_updateFeed.lastUpdated = current.toSecsSinceEpoch();
+    m_updateFeed.lastHash = QString::fromLatin1(QCryptographicHash::hash(m_data, QCryptographicHash::Sha256).toHex());
 
     // Retrieve "other" fields; this will include the "itunes" tags
     QMultiMap<QString, QDomElement> otherItems = feed->additionalProperties();
 
-    query.prepare(QStringLiteral(
-        "UPDATE Feeds SET name=:name, image=:image, link=:link, description=:description, lastUpdated=:lastUpdated, dirname=:dirname WHERE url=:url;"));
-    query.bindValue(QStringLiteral(":name"), feed->title());
-    query.bindValue(QStringLiteral(":url"), m_url);
-    query.bindValue(QStringLiteral(":link"), feed->link());
-    query.bindValue(QStringLiteral(":description"), feed->description());
-
-    QDateTime current = QDateTime::currentDateTime();
-    query.bindValue(QStringLiteral(":lastUpdated"), current.toSecsSinceEpoch());
-
-    QString image;
     // First try the itunes tags, if not, fall back to regular image tag
     if (otherItems.value(QStringLiteral("http://www.itunes.com/dtds/podcast-1.0.dtdimage")).hasAttribute(QStringLiteral("href"))) {
-        image = otherItems.value(QStringLiteral("http://www.itunes.com/dtds/podcast-1.0.dtdimage")).attribute(QStringLiteral("href"));
+        m_updateFeed.image = otherItems.value(QStringLiteral("http://www.itunes.com/dtds/podcast-1.0.dtdimage")).attribute(QStringLiteral("href"));
     } else {
-        image = feed->image()->url();
+        m_updateFeed.image = feed->image()->url();
     }
 
-    if (image.startsWith(QStringLiteral("/")))
-        image = QUrl(m_url).adjusted(QUrl::RemovePath).toString() + image;
-    query.bindValue(QStringLiteral(":image"), image);
+    if (m_updateFeed.image.startsWith(QStringLiteral("/"))) {
+        m_updateFeed.image = QUrl(m_url).adjusted(QUrl::RemovePath).toString() + m_updateFeed.image;
+    }
 
     // if the title has changed, we need to rename the corresponding enclosure
     // download directory name and move the files
-    m_dirname = oldDirname;
-    if (oldName != feed->title() || oldDirname.isEmpty() || m_isNewFeed) {
-        QString generatedDirname = generateFeedDirname(feed->title());
-        if (generatedDirname != oldDirname) {
-            m_dirname = generatedDirname;
+    if (m_feed.name != m_updateFeed.name || m_feed.dirname.isEmpty() || m_feed.isNew) {
+        QString generatedDirname = generateFeedDirname(m_updateFeed.name);
+        if (generatedDirname != m_feed.dirname) {
+            m_updateFeed.dirname = generatedDirname;
             QString enclosurePath = StorageManager::instance().enclosureDirPath();
-            if (QDir(enclosurePath + oldDirname).exists()) {
-                QDir().rename(enclosurePath + oldDirname, enclosurePath + m_dirname);
+            if (QDir(enclosurePath + m_feed.dirname).exists()) {
+                QDir().rename(enclosurePath + m_feed.dirname, enclosurePath + m_updateFeed.dirname);
             } else {
-                QDir().mkpath(enclosurePath + m_dirname);
+                QDir().mkpath(enclosurePath + m_updateFeed.dirname);
             }
         }
     }
-    query.bindValue(QStringLiteral(":dirname"), m_dirname);
 
-    // Do the actual database UPDATE of this feed
-    Database::execute(query);
+    QSqlQuery query(QSqlDatabase::database(m_url));
+    query.prepare(QStringLiteral(
+        "UPDATE Feeds SET name=:name, image=:image, link=:link, description=:description, lastUpdated=:lastUpdated, dirname=:dirname WHERE url=:url;"));
+    query.bindValue(QStringLiteral(":name"), m_updateFeed.name);
+    query.bindValue(QStringLiteral(":url"), m_updateFeed.url);
+    query.bindValue(QStringLiteral(":link"), m_updateFeed.link);
+    query.bindValue(QStringLiteral(":description"), m_updateFeed.description);
+    query.bindValue(QStringLiteral(":lastUpdated"), m_updateFeed.lastUpdated);
+    query.bindValue(QStringLiteral(":image"), m_updateFeed.image);
+    query.bindValue(QStringLiteral(":dirname"), m_updateFeed.dirname);
+    // we only write the new lastHash to the database after entries etc. have
+    // all been updated!
+    dbExecute(query);
+    query.clear(); // make sure this query is not blocking anything anymore
 
     // Now that we have the feed details, we make vectors of the data that's
     // already in the database relating to this feed
@@ -160,7 +136,7 @@ void UpdateFeedJob::processFeed(Syndication::FeedPtr feed)
     // we can't check for duplicates and we'll keep adding more of the same!
     query.prepare(QStringLiteral("SELECT * FROM Entries WHERE feed=:feed;"));
     query.bindValue(QStringLiteral(":feed"), m_url);
-    Database::execute(query);
+    dbExecute(query);
     while (query.next()) {
         EntryDetails entryDetails;
         entryDetails.feed = m_url;
@@ -176,10 +152,11 @@ void UpdateFeedJob::processFeed(Syndication::FeedPtr feed)
         entryDetails.image = query.value(QStringLiteral("image")).toString();
         m_entries += entryDetails;
     }
+    query.clear();
 
     query.prepare(QStringLiteral("SELECT * FROM Enclosures WHERE feed=:feed;"));
     query.bindValue(QStringLiteral(":feed"), m_url);
-    Database::execute(query);
+    dbExecute(query);
     while (query.next()) {
         EnclosureDetails enclosureDetails;
         enclosureDetails.feed = m_url;
@@ -193,10 +170,11 @@ void UpdateFeedJob::processFeed(Syndication::FeedPtr feed)
         enclosureDetails.downloaded = Enclosure::dbToStatus(query.value(QStringLiteral("downloaded")).toInt());
         m_enclosures += enclosureDetails;
     }
+    query.clear();
 
     query.prepare(QStringLiteral("SELECT * FROM Authors WHERE feed=:feed;"));
     query.bindValue(QStringLiteral(":feed"), m_url);
-    Database::execute(query);
+    dbExecute(query);
     while (query.next()) {
         AuthorDetails authorDetails;
         authorDetails.feed = m_url;
@@ -206,10 +184,11 @@ void UpdateFeedJob::processFeed(Syndication::FeedPtr feed)
         authorDetails.email = query.value(QStringLiteral("email")).toString();
         m_authors += authorDetails;
     }
+    query.clear();
 
     query.prepare(QStringLiteral("SELECT * FROM Chapters WHERE feed=:feed;"));
     query.bindValue(QStringLiteral(":feed"), m_url);
-    Database::execute(query);
+    dbExecute(query);
     while (query.next()) {
         ChapterDetails chapterDetails;
         chapterDetails.feed = m_url;
@@ -220,13 +199,13 @@ void UpdateFeedJob::processFeed(Syndication::FeedPtr feed)
         chapterDetails.image = query.value(QStringLiteral("image")).toString();
         m_chapters += chapterDetails;
     }
+    query.clear();
 
-    query.clear(); // make sure this query is not blocking anything anymore
-
+    bool authorsChanged = false;
     // Process feed authors
     if (feed->authors().count() > 0) {
         for (auto &author : feed->authors()) {
-            processAuthor(QLatin1String(""), author->name(), QLatin1String(""), QLatin1String(""));
+            authorsChanged = authorsChanged || processAuthor(QLatin1String(""), author->name(), QLatin1String(""), QLatin1String(""));
         }
     } else {
         // Try to find itunes fields if plain author doesn't exist
@@ -246,21 +225,32 @@ void UpdateFeedJob::processFeed(Syndication::FeedPtr feed)
             qCDebug(kastsFetcher) << "authorname" << authorname;
         }
         if (!authorname.isEmpty()) {
-            processAuthor(QLatin1String(""), authorname, QLatin1String(""), authoremail);
+            authorsChanged = authorsChanged || processAuthor(QLatin1String(""), authorname, QLatin1String(""), authoremail);
         }
     }
 
     qCDebug(kastsFetcher) << "Updated feed details:" << feed->title();
 
-    // TODO: Only emit signal if the details have really changed
-    Q_EMIT feedDetailsUpdated(m_url, feed->title(), image, feed->link(), feed->description(), current, m_dirname);
+    // check if any field has changed and only emit signal if there are changes
+    bool hasFeedBeenUpdated = false;
+    if (m_updateFeed.name != m_feed.name || m_updateFeed.link != m_feed.link || m_updateFeed.image != m_feed.image
+        || m_updateFeed.description != m_feed.description || authorsChanged) {
+        hasFeedBeenUpdated = true;
+    } else {
+        hasFeedBeenUpdated = false;
+    }
+
+    if (hasFeedBeenUpdated) {
+        Q_EMIT feedDetailsUpdated(m_url, m_updateFeed.name, m_updateFeed.image, m_updateFeed.link, m_updateFeed.description, current, m_updateFeed.dirname);
+    }
 
     if (m_abort)
         return;
 
     // Now deal with the entries, enclosures, entry authors and chapter marks
     bool updatedEntries = false;
-    for (const auto &entry : feed->items()) {
+    const auto items = feed->items();
+    for (const auto &entry : items) {
         if (m_abort)
             return;
 
@@ -270,26 +260,10 @@ void UpdateFeedJob::processFeed(Syndication::FeedPtr feed)
 
     writeToDatabase();
 
-    if (m_isNewFeed) {
-        // Finally, reset the new flag to false now that the new feed has been
-        // fully processed.  If we would reset the flag sooner, then too many
-        // episodes will get flagged as new if the initial import gets
-        // interrupted somehow.
-        query.prepare(QStringLiteral("UPDATE Feeds SET new=:new WHERE url=:url;"));
-        query.bindValue(QStringLiteral(":url"), m_url);
-        query.bindValue(QStringLiteral(":new"), false);
-        Database::execute(query);
-    }
-
-    if (m_oldHash != m_newHash) {
-        query.prepare(QStringLiteral("UPDATE Feeds SET lastHash=:lastHash WHERE url=:url;"));
-        query.bindValue(QStringLiteral(":url"), m_url);
-        query.bindValue(QStringLiteral(":lastHash"), m_newHash);
-        Database::execute(query);
-    }
-
-    if (updatedEntries || m_isNewFeed)
+    if (updatedEntries || m_feed.isNew) {
         Q_EMIT feedUpdated(m_url);
+    }
+
     qCDebug(kastsFetcher) << "done processing feed" << feed;
 }
 
@@ -302,7 +276,8 @@ bool UpdateFeedJob::processEntry(Syndication::ItemPtr entry)
     EntryDetails currentEntry;
 
     // check against existing entries and the list of new entries
-    for (const EntryDetails &entryDetails : (m_entries + m_newEntries)) {
+    const auto allEntries = m_entries + m_newEntries;
+    for (const EntryDetails &entryDetails : allEntries) {
         if (entryDetails.id == entry->id()) {
             isNewEntry = false;
             currentEntry = entryDetails;
@@ -317,7 +292,8 @@ bool UpdateFeedJob::processEntry(Syndication::ItemPtr entry)
     // Retrieve "other" fields; this will include the "itunes" tags
     QMultiMap<QString, QDomElement> otherItems = entry->additionalProperties();
 
-    for (const QString &key : otherItems.uniqueKeys()) {
+    const auto uniqueKeys = otherItems.uniqueKeys();
+    for (const QString &key : uniqueKeys) {
         qCDebug(kastsFetcher) << "other elements";
         qCDebug(kastsFetcher) << key << otherItems.value(key).tagName();
     }
@@ -330,8 +306,8 @@ bool UpdateFeedJob::processEntry(Syndication::ItemPtr entry)
     entryDetails.updated = static_cast<int>(entry->dateUpdated());
     entryDetails.link = entry->link();
     entryDetails.hasEnclosure = (entry->enclosures().length() > 0);
-    entryDetails.read = m_isNewFeed ? m_markUnreadOnNewFeed : false; // if new feed, then check settings
-    entryDetails.isNew = !m_isNewFeed; // if new feed, then mark none as new
+    entryDetails.read = m_feed.isNew ? m_markUnreadOnNewFeed : false; // if new feed, then check settings
+    entryDetails.isNew = !m_feed.isNew; // if new feed, then mark none as new
 
     // Take the longest text, either content or description
     if (entry->content().length() > entry->description().length()) {
@@ -369,7 +345,8 @@ bool UpdateFeedJob::processEntry(Syndication::ItemPtr entry)
 
     // Process authors
     if (entry->authors().count() > 0) {
-        for (const auto &author : entry->authors()) {
+        const auto authors = entry->authors();
+        for (const auto &author : authors) {
             isUpdateDependencies = isUpdateDependencies | processAuthor(entry->id(), author->name(), author->uri(), author->email());
         }
     } else {
@@ -393,7 +370,7 @@ bool UpdateFeedJob::processEntry(Syndication::ItemPtr entry)
                     startParts.removeLast();
                 }
                 int startInt = 0;
-                for (QString part : startParts) {
+                for (const QString &part : std::as_const(startParts)) {
                     // strip off decimal point if it's present
                     startInt = part.split(QStringLiteral("."))[0].toInt() + startInt * 60;
                 }
@@ -422,7 +399,8 @@ bool UpdateFeedJob::processAuthor(const QString &entryId, const QString &authorN
     AuthorDetails currentAuthor;
 
     // check against existing authors already in database
-    for (const AuthorDetails &authorDetails : (m_authors + m_newAuthors)) {
+    const auto allAuthors = m_authors + m_newAuthors;
+    for (const AuthorDetails &authorDetails : allAuthors) {
         if ((authorDetails.id == entryId) && (authorDetails.name == authorName)) {
             isNewAuthor = false;
             currentAuthor = authorDetails;
@@ -459,7 +437,8 @@ bool UpdateFeedJob::processEnclosure(Syndication::EnclosurePtr enclosure, const
     EnclosureDetails currentEnclosure;
 
     // check against existing enclosures already in database
-    for (const EnclosureDetails &enclosureDetails : (m_enclosures + m_newEnclosures)) {
+    const auto allEnclosures = m_enclosures + m_newEnclosures;
+    for (const EnclosureDetails &enclosureDetails : allEnclosures) {
         if (enclosureDetails.id == newEntry.id) {
             isNewEnclosure = false;
             currentEnclosure = enclosureDetails;
@@ -489,8 +468,8 @@ bool UpdateFeedJob::processEnclosure(Syndication::EnclosurePtr enclosure, const
 
         // Check if entry title or enclosure URL has changed
         if (newEntry.title != oldEntry.title) {
-            QString oldFilename = StorageManager::instance().enclosurePath(oldEntry.title, currentEnclosure.url, m_dirname);
-            QString newFilename = StorageManager::instance().enclosurePath(newEntry.title, enclosureDetails.url, m_dirname);
+            QString oldFilename = StorageManager::instance().enclosurePath(oldEntry.title, currentEnclosure.url, m_updateFeed.dirname);
+            QString newFilename = StorageManager::instance().enclosurePath(newEntry.title, enclosureDetails.url, m_updateFeed.dirname);
 
             if (oldFilename != newFilename) {
                 if (currentEnclosure.url == enclosureDetails.url) {
@@ -519,7 +498,8 @@ bool UpdateFeedJob::processChapter(const QString &entryId, const int &start, con
     ChapterDetails currentChapter;
 
     // check against existing enclosures already in database
-    for (const ChapterDetails &chapterDetails : (m_chapters + m_newChapters)) {
+    const auto allChapters = m_chapters + m_newChapters;
+    for (const ChapterDetails &chapterDetails : allChapters) {
         if ((chapterDetails.id == entryId) && (chapterDetails.start == start)) {
             isNewChapter = false;
             currentChapter = chapterDetails;
@@ -554,12 +534,12 @@ void UpdateFeedJob::writeToDatabase()
 {
     QSqlQuery writeQuery(QSqlDatabase::database(m_url));
 
-    Database::transaction(m_url);
+    dbTransaction();
 
     // new entries
     writeQuery.prepare(
         QStringLiteral("INSERT INTO Entries VALUES (:feed, :id, :title, :content, :created, :updated, :link, :read, :new, :hasEnclosure, :image, :favorite);"));
-    for (const EntryDetails &entryDetails : m_newEntries) {
+    for (const EntryDetails &entryDetails : std::as_const(m_newEntries)) {
         writeQuery.bindValue(QStringLiteral(":feed"), entryDetails.feed);
         writeQuery.bindValue(QStringLiteral(":id"), entryDetails.id);
         writeQuery.bindValue(QStringLiteral(":title"), entryDetails.title);
@@ -572,14 +552,15 @@ void UpdateFeedJob::writeToDatabase()
         writeQuery.bindValue(QStringLiteral(":new"), entryDetails.isNew);
         writeQuery.bindValue(QStringLiteral(":image"), entryDetails.image);
         writeQuery.bindValue(QStringLiteral(":favorite"), false);
-        Database::execute(writeQuery);
+        dbExecute(writeQuery);
     }
+    writeQuery.clear();
 
     // update entries
     writeQuery.prepare(
         QStringLiteral("UPDATE Entries SET title=:title, content=:content, created=:created, updated=:updated, link=:link, hasEnclosure=:hasEnclosure, "
                        "image=:image WHERE id=:id AND feed=:feed;"));
-    for (const EntryDetails &entryDetails : m_updateEntries) {
+    for (const EntryDetails &entryDetails : std::as_const(m_updateEntries)) {
         writeQuery.bindValue(QStringLiteral(":feed"), entryDetails.feed);
         writeQuery.bindValue(QStringLiteral(":id"), entryDetails.id);
         writeQuery.bindValue(QStringLiteral(":title"), entryDetails.title);
@@ -589,34 +570,37 @@ void UpdateFeedJob::writeToDatabase()
         writeQuery.bindValue(QStringLiteral(":link"), entryDetails.link);
         writeQuery.bindValue(QStringLiteral(":hasEnclosure"), entryDetails.hasEnclosure);
         writeQuery.bindValue(QStringLiteral(":image"), entryDetails.image);
-        Database::execute(writeQuery);
+        dbExecute(writeQuery);
     }
+    writeQuery.clear();
 
     // new authors
     writeQuery.prepare(QStringLiteral("INSERT INTO Authors VALUES(:feed, :id, :name, :uri, :email);"));
-    for (const AuthorDetails &authorDetails : m_newAuthors) {
+    for (const AuthorDetails &authorDetails : std::as_const(m_newAuthors)) {
         writeQuery.bindValue(QStringLiteral(":feed"), authorDetails.feed);
         writeQuery.bindValue(QStringLiteral(":id"), authorDetails.id);
         writeQuery.bindValue(QStringLiteral(":name"), authorDetails.name);
         writeQuery.bindValue(QStringLiteral(":uri"), authorDetails.uri);
         writeQuery.bindValue(QStringLiteral(":email"), authorDetails.email);
-        Database::execute(writeQuery);
+        dbExecute(writeQuery);
     }
+    writeQuery.clear();
 
     // update authors
     writeQuery.prepare(QStringLiteral("UPDATE Authors SET uri=:uri, email=:email WHERE feed=:feed AND id=:id AND name=:name;"));
-    for (const AuthorDetails &authorDetails : m_updateAuthors) {
+    for (const AuthorDetails &authorDetails : std::as_const(m_updateAuthors)) {
         writeQuery.bindValue(QStringLiteral(":feed"), authorDetails.feed);
         writeQuery.bindValue(QStringLiteral(":id"), authorDetails.id);
         writeQuery.bindValue(QStringLiteral(":name"), authorDetails.name);
         writeQuery.bindValue(QStringLiteral(":uri"), authorDetails.uri);
         writeQuery.bindValue(QStringLiteral(":email"), authorDetails.email);
-        Database::execute(writeQuery);
+        dbExecute(writeQuery);
     }
+    writeQuery.clear();
 
     // new enclosures
     writeQuery.prepare(QStringLiteral("INSERT INTO Enclosures VALUES (:feed, :id, :duration, :size, :title, :type, :url, :playposition, :downloaded);"));
-    for (const EnclosureDetails &enclosureDetails : m_newEnclosures) {
+    for (const EnclosureDetails &enclosureDetails : std::as_const(m_newEnclosures)) {
         writeQuery.bindValue(QStringLiteral(":feed"), enclosureDetails.feed);
         writeQuery.bindValue(QStringLiteral(":id"), enclosureDetails.id);
         writeQuery.bindValue(QStringLiteral(":duration"), enclosureDetails.duration);
@@ -626,12 +610,13 @@ void UpdateFeedJob::writeToDatabase()
         writeQuery.bindValue(QStringLiteral(":url"), enclosureDetails.url);
         writeQuery.bindValue(QStringLiteral(":playposition"), enclosureDetails.playPosition);
         writeQuery.bindValue(QStringLiteral(":downloaded"), Enclosure::statusToDb(enclosureDetails.downloaded));
-        Database::execute(writeQuery);
+        dbExecute(writeQuery);
     }
+    writeQuery.clear();
 
     // update enclosures
     writeQuery.prepare(QStringLiteral("UPDATE Enclosures SET duration=:duration, size=:size, title=:title, type=:type, url=:url WHERE feed=:feed AND id=:id;"));
-    for (const EnclosureDetails &enclosureDetails : m_updateEnclosures) {
+    for (const EnclosureDetails &enclosureDetails : std::as_const(m_updateEnclosures)) {
         writeQuery.bindValue(QStringLiteral(":feed"), enclosureDetails.feed);
         writeQuery.bindValue(QStringLiteral(":id"), enclosureDetails.id);
         writeQuery.bindValue(QStringLiteral(":duration"), enclosureDetails.duration);
@@ -639,89 +624,143 @@ void UpdateFeedJob::writeToDatabase()
         writeQuery.bindValue(QStringLiteral(":title"), enclosureDetails.title);
         writeQuery.bindValue(QStringLiteral(":type"), enclosureDetails.type);
         writeQuery.bindValue(QStringLiteral(":url"), enclosureDetails.url);
-        Database::execute(writeQuery);
+        dbExecute(writeQuery);
     }
+    writeQuery.clear();
 
     // new chapters
     writeQuery.prepare(QStringLiteral("INSERT INTO Chapters VALUES(:feed, :id, :start, :title, :link, :image);"));
-    for (const ChapterDetails &chapterDetails : m_newChapters) {
+    for (const ChapterDetails &chapterDetails : std::as_const(m_newChapters)) {
         writeQuery.bindValue(QStringLiteral(":feed"), chapterDetails.feed);
         writeQuery.bindValue(QStringLiteral(":id"), chapterDetails.id);
         writeQuery.bindValue(QStringLiteral(":start"), chapterDetails.start);
         writeQuery.bindValue(QStringLiteral(":title"), chapterDetails.title);
         writeQuery.bindValue(QStringLiteral(":link"), chapterDetails.link);
         writeQuery.bindValue(QStringLiteral(":image"), chapterDetails.image);
-        Database::execute(writeQuery);
+        dbExecute(writeQuery);
     }
+    writeQuery.clear();
 
     // update chapters
     writeQuery.prepare(QStringLiteral("UPDATE Chapters SET title=:title, link=:link, image=:image WHERE feed=:feed AND id=:id AND start=:start;"));
-    for (const ChapterDetails &chapterDetails : m_updateChapters) {
+    for (const ChapterDetails &chapterDetails : std::as_const(m_updateChapters)) {
         writeQuery.bindValue(QStringLiteral(":feed"), chapterDetails.feed);
         writeQuery.bindValue(QStringLiteral(":id"), chapterDetails.id);
         writeQuery.bindValue(QStringLiteral(":start"), chapterDetails.start);
         writeQuery.bindValue(QStringLiteral(":title"), chapterDetails.title);
         writeQuery.bindValue(QStringLiteral(":link"), chapterDetails.link);
         writeQuery.bindValue(QStringLiteral(":image"), chapterDetails.image);
-        Database::execute(writeQuery);
+        dbExecute(writeQuery);
     }
+    writeQuery.clear();
 
     // set custom amount of episodes to unread/new if required
-    if (m_isNewFeed && (SettingsManager::self()->markUnreadOnNewFeed() == 1) && (SettingsManager::self()->markUnreadOnNewFeedCustomAmount() > 0)) {
+    if (m_feed.isNew && (SettingsManager::self()->markUnreadOnNewFeed() == 1) && (SettingsManager::self()->markUnreadOnNewFeedCustomAmount() > 0)) {
         writeQuery.prepare(QStringLiteral(
             "UPDATE Entries SET read=:read, new=:new WHERE id in (SELECT id FROM Entries WHERE feed =:feed ORDER BY updated DESC LIMIT :recentUnread);"));
         writeQuery.bindValue(QStringLiteral(":feed"), m_url);
         writeQuery.bindValue(QStringLiteral(":read"), false);
         writeQuery.bindValue(QStringLiteral(":new"), true);
         writeQuery.bindValue(QStringLiteral(":recentUnread"), SettingsManager::self()->markUnreadOnNewFeedCustomAmount());
-        Database::execute(writeQuery);
+        dbExecute(writeQuery);
+        writeQuery.clear();
     }
 
-    if (Database::commit(m_url)) {
+    if (m_feed.isNew) {
+        // Finally, reset the new flag to false now that the new feed has been
+        // fully processed.  If we would reset the flag sooner, then too many
+        // episodes will get flagged as new if the initial import gets
+        // interrupted somehow.
+        writeQuery.prepare(QStringLiteral("UPDATE Feeds SET new=:new WHERE url=:url;"));
+        writeQuery.bindValue(QStringLiteral(":url"), m_url);
+        writeQuery.bindValue(QStringLiteral(":new"), false);
+        dbExecute(writeQuery);
+        writeQuery.clear();
+    }
+
+    if (m_feed.lastHash != m_updateFeed.lastHash) {
+        writeQuery.prepare(QStringLiteral("UPDATE Feeds SET lastHash=:lastHash WHERE url=:url;"));
+        writeQuery.bindValue(QStringLiteral(":url"), m_url);
+        writeQuery.bindValue(QStringLiteral(":lastHash"), m_updateFeed.lastHash);
+        dbExecute(writeQuery);
+        writeQuery.clear();
+    }
+
+    if (dbCommit()) {
         QStringList newIds, updateIds;
 
         // emit signals for new entries
-        for (const EntryDetails &entryDetails : m_newEntries) {
+        for (const EntryDetails &entryDetails : std::as_const(m_newEntries)) {
             if (!newIds.contains(entryDetails.id)) {
                 newIds += entryDetails.id;
             }
         }
 
-        for (const QString &id : newIds) {
+        for (const QString &id : std::as_const(newIds)) {
             Q_EMIT entryAdded(m_url, id);
         }
 
         // emit signals for updated entries or entries with new/updated authors,
         // enclosures or chapters
-        for (const EntryDetails &entryDetails : m_updateEntries) {
+        for (const EntryDetails &entryDetails : std::as_const(m_updateEntries)) {
             if (!updateIds.contains(entryDetails.id) && !newIds.contains(entryDetails.id)) {
                 updateIds += entryDetails.id;
             }
         }
-        for (const EnclosureDetails &enclosureDetails : (m_newEnclosures + m_updateEnclosures)) {
+        const auto allEnclosures = m_newEnclosures + m_updateEnclosures;
+        for (const EnclosureDetails &enclosureDetails : allEnclosures) {
             if (!updateIds.contains(enclosureDetails.id) && !newIds.contains(enclosureDetails.id)) {
                 updateIds += enclosureDetails.id;
             }
         }
-        for (const AuthorDetails &authorDetails : (m_newAuthors + m_updateAuthors)) {
+        const auto allAuthors = m_newAuthors + m_updateAuthors;
+        for (const AuthorDetails &authorDetails : allAuthors) {
             if (!updateIds.contains(authorDetails.id) && !newIds.contains(authorDetails.id)) {
                 updateIds += authorDetails.id;
             }
         }
-        for (const ChapterDetails &chapterDetails : (m_newChapters + m_updateChapters)) {
+        const auto allChapters = m_newChapters + m_updateChapters;
+        for (const ChapterDetails &chapterDetails : allChapters) {
             if (!updateIds.contains(chapterDetails.id) && !newIds.contains(chapterDetails.id)) {
                 updateIds += chapterDetails.id;
             }
         }
 
-        for (const QString &id : updateIds) {
+        for (const QString &id : std::as_const(updateIds)) {
             qCDebug(kastsFetcher) << "updated episode" << id;
             Q_EMIT entryUpdated(m_url, id);
         }
     }
 }
 
-QString UpdateFeedJob::generateFeedDirname(const QString &name) const
+bool UpdateFeedJob::dbExecute(QSqlQuery &query)
+{
+    bool state = Database::executeThread(query);
+
+    if (!state) {
+        Q_EMIT error(Error::Type::Database, QString(), QString(), query.lastError().type(), query.lastQuery(), query.lastError().text());
+    }
+
+    return state;
+}
+
+bool UpdateFeedJob::dbTransaction()
+{
+    // use raw sqlite query to benefit from automatic retries on execute
+    QSqlQuery query(QSqlDatabase::database(m_url));
+    query.prepare(QStringLiteral("BEGIN TRANSACTION;"));
+    return dbExecute(query);
+}
+
+bool UpdateFeedJob::dbCommit()
+{
+    // use raw sqlite query to benefit from automatic retries on execute
+    QSqlQuery query(QSqlDatabase::database(m_url));
+    query.prepare(QStringLiteral("COMMIT TRANSACTION;"));
+    return dbExecute(query);
+}
+
+QString UpdateFeedJob::generateFeedDirname(const QString &name)
 {
     // Generate directory name for enclosures based on feed name
     // NOTE: Any changes here require a database migration!
@@ -731,6 +770,7 @@ QString UpdateFeedJob::generateFeedDirname(const QString &name) const
     QStringList dirNameList;
     QSqlQuery query(QSqlDatabase::database(m_url));
     query.prepare(QStringLiteral("SELECT name FROM Feeds;"));
+    dbExecute(query);
     while (query.next()) {
         dirNameList << query.value(QStringLiteral("name")).toString();
     }
